@@ -117,10 +117,12 @@ Version: 0.2
 
 from . import true_np
 from . import vecvalder_funcs_and_ufuncs as masked_functions
+from . import sparsevecvalder_funcs_and_ufuncs as sparse_masked_functions
 
 from .numpy_masking import AutoDiff
 from .numpy_masking import _active_auto_diffs
 
+from .sparse_numpy_masking import SparseAutoDiff
 
 def get_value_and_jacobians(x):
     """Extracts values and jacobians from the value of a function,
@@ -131,10 +133,18 @@ def get_value_and_jacobians(x):
     import numpy as np
     if isinstance(x, true_np.ndarray):
         x = np.array(x)
-    shared_axes = tuple(slice(None) for _ in x.val.shape)
-
+    
     jacobians = []
     total = 0
+
+    if isinstance(_active_auto_diffs[-1], SparseAutoDiff):
+        for vec in _active_auto_diffs[-1].vecs:
+            jacobians.append(x.der[:, total:total + np.size(vec)])
+            total += np.size(vec)
+
+        return x.val, tuple(jacobians)
+
+    shared_axes = tuple(slice(None) for _ in x.val.shape)
     for vec in _active_auto_diffs[-1].vecs:
         idx = (*shared_axes, slice(total, total + vec.shape[0]), slice(None))
         jacobians.append(
@@ -148,6 +158,8 @@ def get_value_and_jacobian(x):
     import numpy as np
     if isinstance(x, true_np.ndarray):
         x = np.array(x)
+    elif isinstance(_active_auto_diffs[-1], SparseAutoDiff):
+        return x.val, jacobian(x)
     if x.der.shape[-1] == 0:
         assert x.der.shape[2] == 1
         return x.val, true_np.ndarray((-1, 0))
@@ -158,6 +170,10 @@ def jacobian(x):
     import numpy as np
     if isinstance(x, true_np.ndarray):
         x = np.array(x)
+
+    if isinstance(_active_auto_diffs[-1], SparseAutoDiff):
+        return x.der
+
     if x.der.shape[-1] == 0:
         assert x.der.shape[2] == 1
         return true_np.ndarray((x.der.shape[0], 0))
@@ -165,3 +181,20 @@ def jacobian(x):
     i, j, k, n = x.der.shape
     assert j == 1 == n
     return x.der.reshape((i, k))
+
+
+def generate_jacobian_function(f, n=0):
+    def retval(*args, **kwargs):
+        args_before, arg, args_after = args[:n], args[n], args[n+1:]
+        with AutoDiff(arg) as ad_arg:
+            return jacobian(f(*args_before, ad_arg, *args_after, **kwargs))
+    return retval
+
+__all__ = [
+    'AutoDiff',
+    'SparseAutoDiff',
+    'get_value_and_jacobian',
+    'get_value_and_jacobians',
+    'jacobian',
+    'generate_jacobian_function',
+]
